// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  TU Dresden                                                            ==
// ==                                                                        ==
// ==  Institut fr Wissenschaftliches Rechnen                               ==
// ==  Zellescher Weg 12-14                                                  ==
// ==  01069 Dresden                                                         ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  https://gforge.zih.tu-dresden.de/projects/amdis/                      ==
// ==                                                                        ==
// ============================================================================

/** \file RefinementManager3d.h */

#ifndef AMDIS_REFINEMENT_MANAGER_3D_H
#define AMDIS_REFINEMENT_MANAGER_3D_H

namespace AMDiS {

  /** \ingroup Adaption 
   * \brief
   * Implements a RefinementManager for 3-dimensional meshes.
   */
  class RefinementManager3d : public RefinementManager
  {
  public:
    /// Calls base class constructor.
    RefinementManager3d() 
      : RefinementManager()
    {}

    /// destructor 
    virtual ~RefinementManager3d() {}

  protected:
    /// Used by \ref setNewCoords
    int newCoordsFct(ElInfo *el_info);

    /// Implements RefinementManager::setNewCoords
    void setNewCoords();

    /** \brief
     * Gets the elements around the refinement edge with vertices node[0] and
     * node[1]. Refines those elements at this edge that are not compatible 
     * devisible. The function returns 1 if the domain's boundary is reached
     * while looping around the refinement edge, otherwise 0.
     * 
     * \param[in] direction   Determines the direction of the first neighbour
     *  
     */
    int getRefinePatch(ElInfo **el_info, 
		       DegreeOfFreedom *edge[2], 
		       int direction,
		       RCNeighbourList* refineList, 
		       int *n_neigh);

    /// Refines all elements in the patch.
    DegreeOfFreedom refinePatch(DegreeOfFreedom *edge[2], RCNeighbourList* refineList,
				int n_neigh, bool bound);

    /// Implements RefinementManager::refineFunction.
    ElInfo* refineFunction(ElInfo* el_info);

    /// Refines one Tetrahedron.
    void bisectTetrahedron(RCNeighbourList* refineList, int index,
			   DegreeOfFreedom *dof[3], DegreeOfFreedom *edge[2]);

    /// Used by \ref bisectTetrahedron
    void fillPatchConnectivity(RCNeighbourList* ref_list, int index);

  private:
    /// Refinement patch
    static RCNeighbourList* refList;
  };

}

#endif // AMDIS_REFINEMENT_MANAGER_3D_H
